/* Copyright Gerben Wierda, 1998 */
/* No Warranty */

#include <stdio.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>
#include <libc.h>
#include <regex.h>

extern char *optarg;
extern int optind;

#define SECSINDAY (24*3600)
#define SECSINHOUR 3600
#define SECSINMINUTE 60

typedef enum { unknown, fromnow, daytime} TYPEOFSETTING;

void main( int argc, char *argv[])
{
    long now = time( 0);
    struct tm *tmsp = localtime( &now);
    TYPEOFSETTING type = unknown;
    long daysfromnow = 0; /* default today */
    long timeofday = 0 * SECSINHOUR; /* default @ 0AM */
    long secsfromnow = 15 * SECSINMINUTE; /* default 0:15 from now*/
    int c = EOF, errflag = 0;
    int quiet = 0;
    int dontexecute = 0;
    char *powerdownarg = NULL;
    int powerdown = 0;
    long newtime;
    char command[128];

    if (geteuid() != 0) {
	fprintf( stderr, "You're only allowed to run this program as root.\n");
	exit( 1);
    }
    
    while ((c = getopt( argc, argv, "d:s:h:nqp:")) != EOF) {
	switch (c) {
	case 'p':
	    powerdown = 1;
	    powerdownarg = calloc( strlen( optarg) + 1, 1);
	    strcpy( powerdownarg, optarg);
	    break;
	case 'n':
	    dontexecute = 1;
	    break;
	case 'd':
	    daysfromnow = atol( optarg);
	    type = daytime;
	    break;
	case 's':
	    secsfromnow = atol( optarg);
	    type = fromnow;
	    break;
	case 'h':
	    if (!recmp( "[0-9][0-9]:[0-9][0-9]", optarg)) {
		/*
		 * ':' found, the part before it is hours, the part after is
		 * minutes }
		 */
		type = daytime;
		optarg[2] = 0;
		timeofday = atol( optarg) * SECSINHOUR +
			    atol( optarg + 3) * SECSINMINUTE;
	    }
	    else {
		fprintf( stderr, "Illegal time of day value %s, should ne HH:MM\n", optarg);
		errflag++;
	    }
	    break;
	case 'q':
	    quiet = 1;
	    break;
	default:
	    errflag++;
	    break;
	}
    }
    if (errflag) {
	fprintf( stderr, "Usage: %s [-n] [([-d #] [-h HH:MM]) | [-s #]] [-q]\n",
		 argv[0]);
	exit( 1);
    }

    if (type == unknown) {
	type = fromnow;
    }

    if (type == fromnow) {
	newtime = now + secsfromnow;
    }
    else {
	newtime = (((now / SECSINDAY) + daysfromnow) *
		   SECSINDAY) + timeofday - tmsp->tm_gmtoff;
    }

    if (!quiet) {
	if (dontexecute) {
	    printf( "-- Commands will not be executed --\n");
	}
	printf( "Now = %s", asctime( localtime( &now)));
	printf( "Boottime = %s", asctime( localtime( &newtime)));
	if (powerdown) {
	    printf( "Automatic immediate powerdown requested at \"%s\"\n",
		    powerdownarg);
	}
    }
    fflush( stdout);
    fflush( stderr);
    fclose( stdout);
    fclose( stderr);


    if (!dontexecute) {
	sprintf( command, "/usr/lib/Preferences/clock_chip -T %ld", newtime);
	system( command);
	sprintf( command, "/usr/lib/Preferences/clock_chip -A 1");
	system( command);
	if (powerdown) {
	    sprintf( command, "/usr/etc/shutdown -h -p %s", powerdownarg);
	    system( command);
	}
    }
}
